@@ -24,8 +24,10 @@
 #include <glib.h>
 #include "callback.h"
 
+void ai_panic(const char *message);
 void ai_wait(void);
 void ai_chat(const char *message);
 void greedy_init(int argc, char **argv);
+void lobbybot_init(int argc, char **argv);
 
 #endif
