@@ -128,7 +128,7 @@ static void lobbybot_chat_parser(gint player_num, const gchar * chat)
 	if (!strncmp(chat, "/news", 5)) {
 		ai_chat(N_("The last released version of Pioneers is"));
 		/* Update this string when releasing a new version */
-		ai_chat("0.11.3");
+		ai_chat("0.12.1");
 		return;
 	}
 }
@@ -140,8 +140,10 @@ static void hash_data_free(gpointer data)
 
 static void lobbybot_turn(G_GNUC_UNUSED gint player_num)
 {
-	ai_chat(N_
-		("The game is starting. I'm not needed anymore. Goodbye."));
+	ai_chat(
+		       /* The lobbybot leaves when a game is starting */
+		       N_(""
+			  "The game is starting. I'm not needed anymore. Goodbye."));
 	cb_disconnect();
 }
 
