@@ -36,8 +36,10 @@ static void player_setup(Player * player, int playernum,
 			 const gchar * name, gboolean force_viewer);
 static Player *player_by_name(Game * game, char *name);
 
+#define tournament_minute 1000 * 1
 
 /** Find a free number for a connecting player.
+ * The number has not been used before.
  *  @param game The game
  *  @param force_viewer The connecting player must be a viewer
  */
@@ -56,15 +58,12 @@ static gint next_free_player_num(Game * game, gboolean force_viewer)
 		     list != NULL; list = g_list_next(list)) {
 			Player *player = list->data;
 			if (player->num >= 0
-			    && !player_is_viewer(game, player->num)
-			    && (!player->disconnected
-				|| player->sm->use_cache)) {
+			    && !player_is_viewer(game, player->num)) {
 				player_taken[player->num] = TRUE;
 				--available;
 			}
 		}
 		playerlist_dec_use_count(game);
-
 		if (available > 0) {
 			gint skip;
 			if (game->random_order) {
@@ -197,19 +196,30 @@ static gboolean mode_unhandled(Player * player, gint event)
 static gboolean tournament_start_cb(gpointer data)
 {
 	int i;
-	Player *player = (Player *) data;
-	Game *game = player->game;
+	Game *game = (Game *) data;
 
 	/* if game already started */
 	if (game->num_players == game->params->num_players)
 		return FALSE;
 
-	player_broadcast(player, PB_SILENT, FIRST_VERSION, LATEST_VERSION,
-			 "NOTE %s\n",
-			 N_("Game starts, adding computer players"));
+	if (game->num_players == 0) {
+		player_broadcast(player_none(game), PB_SILENT,
+				 FIRST_VERSION, LATEST_VERSION,
+				 "NOTE %s\n",
+				 N_
+				 ("The last player left, the "
+				  "tournament timer is reset."));
+		game->tournament_countdown = game->params->tournament_time;
+	}
+
+	player_broadcast(player_none(game), PB_SILENT, FIRST_VERSION,
+			 LATEST_VERSION, "NOTE %s\n",
+			 N_("Game starts, adding computer players."));
+	game->tournament_timer = 0;
+
 	/* add computer players to start game */
 	for (i = game->num_players; i < game->params->num_players; i++) {
-		new_computer_player(NULL, game->server_port, TRUE);
+		add_computer_player(game, TRUE);
 	}
 
 	return FALSE;
@@ -221,32 +231,156 @@ static gboolean tournament_start_cb(gpointer data)
  */
 static gboolean talk_about_tournament_cb(gpointer data)
 {
-	Player *player = (Player *) data;
-	Game *game = player->game;
+	Game *game = (Game *) data;
 	const gchar *message;
 
 	/* if game already started */
 	if (game->num_players == game->params->num_players)
 		return FALSE;
 
-	/* FIXME: Use ngettext here? */
-	message = game->params->tournament_time != 1 ?
+	if (game->num_players == 0) {
+		if (game->tournament_timer != 0) {
+			player_broadcast(player_none(game), PB_SILENT,
+					 FIRST_VERSION, LATEST_VERSION,
+					 "NOTE %s\n",
+					 N_
+					 ("The last player left, the "
+					  "tournament timer is reset."));
+			game->tournament_countdown =
+			    game->params->tournament_time;
+			g_source_remove(game->tournament_timer);
+			game->tournament_timer = 0;
+		}
+		return FALSE;
+	}
+
+	/* ngettext can not be used here,
+	 * because the string must be sent untranslated */
+	message = game->tournament_countdown != 1 ?
 	    N_("The game starts in %s minutes.") :
 	    N_("The game starts in %s minute.");
 
-	player_broadcast(player, PB_SILENT, FIRST_VERSION, LATEST_VERSION,
-			 "NOTE1 %d|%s\n",
-			 game->params->tournament_time, message);
-	game->params->tournament_time--;
+	player_broadcast(player_none(game), PB_SILENT, FIRST_VERSION,
+			 LATEST_VERSION, "NOTE1 %d|%s\n",
+			 game->tournament_countdown, message);
+	game->tournament_countdown--;
 
-	if (game->params->tournament_time > 0)
-		g_timeout_add(1000 * 60,
-			      &talk_about_tournament_cb, player);
+	if (game->tournament_countdown > 0)
+		g_timeout_add(tournament_minute,
+			      &talk_about_tournament_cb, game);
 
 	return FALSE;
 }
 
-Player *player_new(Game * game, int fd, gchar * location)
+/** Generate a name for a computer player.
+ * The name will be unique for the game.
+*/
+static gchar *generate_name_for_computer_player(Game * game)
+{
+	gchar *filename;
+	FILE *stream;
+	gchar *line;
+	gchar *name = NULL;
+	int num = 1;
+
+	filename =
+	    g_build_filename(get_pioneers_dir(), "computer_names", NULL);
+	stream = fopen(filename, "r");
+	if (!stream) {
+		g_warning("Unable to open %s", filename);
+		/* Default name for the AI when the computer_names file
+		 * is not found or empty.
+		 */
+	} else {
+		while (read_line_from_file(&line, stream)) {
+			if (player_by_name(game, line) == NULL) {
+				if (g_random_int_range(0, num) == 0) {
+					if (name)
+						g_free(name);
+					name = g_strdup(line);
+				}
+				num++;
+			}
+		}
+		fclose(stream);
+		if (num == 1) {
+			g_warning("Empty file or all names taken: %s",
+				  filename);
+		}
+	}
+	g_free(filename);
+
+	if (name == NULL) {
+		gint counter = 2;
+
+		/* Default name for the AI when the computer_names file
+		 * is not found or empty.
+		 */
+		name = g_strdup(_("Computer Player"));
+
+		while (player_by_name(game, name) != NULL) {
+			g_free(name);
+			name =
+			    g_strdup_printf("%s (%d)",
+					    _("Computer Player"),
+					    counter++);
+		}
+	}
+
+	return name;
+}
+
+/** Add a new computer player (disconnected)
+ */
+gchar *player_new_computer_player(Game * game)
+{
+	Player *player;
+	gchar *name;
+
+	name = generate_name_for_computer_player(game);
+	player = player_new(game, name, "square");
+	player->disconnected = TRUE;
+	sm_goto(player->sm, (StateFunc) mode_idle);
+	return name;
+}
+
+/** Allocate a new Player struct.
+ *  The StateMachine is not initialized.
+ *   */
+Player *player_new(Game * game, const gchar * name, const gchar * style)
+{
+	Player *player;
+	StateMachine *sm;
+
+	player = g_malloc0(sizeof(*player));
+	sm = player->sm = sm_new(player);
+
+	sm_global_set(sm, (StateFunc) mode_global);
+	sm_unhandled_set(sm, (StateFunc) mode_unhandled);
+
+	player->game = game;
+	player->location = g_strdup("not connected");
+	player->devel = deck_new(game->params);
+	game->player_list = g_list_append(game->player_list, player);
+	player->num = -1;
+	player->chapel_played = 0;
+	player->univ_played = 0;
+	player->gov_played = 0;
+	player->libr_played = 0;
+	player->market_played = 0;
+	player->islands_discovered = 0;
+	player->disconnected = FALSE;
+	player->name = g_strdup(name);
+	player->style = g_strdup(style);
+	player->special_points = NULL;
+	player->special_points_next_id = 0;
+
+	driver->player_change(game);
+
+	return player;
+}
+
+Player *player_new_connection(Game * game, int fd, const gchar * location)
 {
 	gchar name[100];
 	gint i;
@@ -287,50 +421,17 @@ Player *player_new(Game * game, int fd, gchar * location)
 		return NULL;
 	}
 
-	player = g_malloc0(sizeof(*player));
-	sm = player->sm = sm_new(player);
-
-	sm_global_set(sm, (StateFunc) mode_global);
-	sm_unhandled_set(sm, (StateFunc) mode_unhandled);
+	player = player_new(game, name, "square");
+	sm = player->sm;
 	sm_use_fd(sm, fd, TRUE);
+	g_free(player->location);
+	player->location = g_strdup(location);
 
 	/* Cache messages of the game in progress until all intial 
 	 * messages have been sent
 	 */
 	sm_set_use_cache(sm, TRUE);
 
-	player->game = game;
-	player->location = g_strdup(location);
-	player->devel = deck_new(game->params);
-	game->player_list = g_list_append(game->player_list, player);
-	player->num = -1;
-	player->chapel_played = 0;
-	player->univ_played = 0;
-	player->gov_played = 0;
-	player->libr_played = 0;
-	player->market_played = 0;
-	player->islands_discovered = 0;
-	player->disconnected = FALSE;
-	player->name = g_strdup(name);
-	player->style = g_strdup("square");
-	player->special_points = NULL;
-	player->special_points_next_id = 0;
-
-	if (game->params->tournament_time > 0) {
-		/* if first player in and this is a tournament start the timer */
-		if (game->num_players == 0) {
-			g_timeout_add(game->params->tournament_time * 60 *
-				      1000 + 500, &tournament_start_cb,
-				      player);
-			g_timeout_add(1000, &talk_about_tournament_cb,
-				      player);
-		} else {
-			player_send_uncached(player, FIRST_VERSION,
-					     LATEST_VERSION, "NOTE %s\n",
-					     N_
-					     ("This game will start soon."));
-		}
-	}
 
 	sm_goto(sm, (StateFunc) mode_check_version);
 
@@ -355,8 +456,8 @@ static void player_set_name_real(Player * player, gchar * name,
 		 * (on error the AI would disconnect) */
 		player_send(player, FIRST_VERSION, LATEST_VERSION,
 			    "NOTE %s\n",
-			    N_
-			    ("Name not changed: new name is already in use"));
+			    N_(""
+			       "Name not changed: new name is already in use"));
 		return;
 	}
 
@@ -483,9 +584,8 @@ void player_archive(Player * player)
 	/* If the player was in the middle of a trade, pop the state
 	   machine and inform others as necessary */
 	state = sm_current(player->sm);
-	if (state == (StateFunc) mode_wait_quote_exit) {
-		/* Fake the acknowledgement */
-		sm_pop(player->sm);
+	if (state == (StateFunc) mode_domestic_quote_rejected) {
+		/* No special actions needed */
 	} else if (state == (StateFunc) mode_domestic_quote) {
 		/* Retract all quotes */
 		for (;;) {
@@ -542,7 +642,17 @@ void player_revive(Player * newp, char *name)
 		}
 		playerlist_dec_use_count(game);
 	}
-	/* if not, see if any disconnected player exists */
+	/* if not, try to find an unused player number */
+	if (current == NULL) {
+		gint num;
+
+		num = next_free_player_num(game, FALSE);
+		if (num < game->params->num_players) {
+			player_setup(newp, -1, name, FALSE);
+			return;
+		}
+	}
+	/* if not, try to take over another disconnected player */
 	if (current == NULL) {
 		playerlist_inc_use_count(game);
 		for (current = game->player_list; current != NULL;
@@ -616,7 +726,10 @@ void player_revive(Player * newp, char *name)
 	newp->sm->stack_ptr = p->sm->stack_ptr;
 	newp->sm->current_state = p->sm->current_state;
 
-	sm_push(newp->sm, (StateFunc) mode_pre_game);
+	if (sm_current(newp->sm) != (StateFunc) mode_pre_game)
+		sm_push(newp->sm, (StateFunc) mode_pre_game);
+	else
+		sm_goto(newp->sm, (StateFunc) mode_pre_game);
 
 	/* Copy longest road and largest army */
 	if (game->longest_road == p)
@@ -688,10 +801,12 @@ gboolean mode_viewer(Player * player, gint event)
 		player_broadcast(player, PB_ALL, FIRST_VERSION,
 				 LATEST_VERSION, "was viewer %d\n",
 				 player->num);
-		player_setup(player, player->num, player->name, FALSE);
+		sm_set_use_cache(player->sm, TRUE);
+		player_setup(player, -1, player->name, FALSE);
 		sm_goto(sm, (StateFunc) mode_pre_game);
 		return TRUE;
 	}
+	sm_set_use_cache(player->sm, TRUE);
 	player_revive(player, player->name);
 	return TRUE;
 }
@@ -720,6 +835,8 @@ static const gchar *client_version_type_to_string(ClientVersionType cvt)
 		return "0.10";
 	case V0_11:
 		return "0.11";
+	case V0_12:
+		return "0.12";
 	}
 	g_return_val_if_reached("");
 }
@@ -772,8 +889,8 @@ static gboolean mode_check_version(Player * player, gint event)
 						     LATEST_VERSION,
 						     "NOTE1 %s|%s\n",
 						     mismatch,
-						     N_
-						     ("Version mismatch: %s"));
+						     N_(""
+							"Version mismatch: %s"));
 				g_free(mismatch);
 				sm_goto(sm, (StateFunc) mode_bad_version);
 			}
@@ -787,6 +904,32 @@ static gboolean mode_check_version(Player * player, gint event)
 	return FALSE;
 }
 
+static void start_tournament_mode(Player * player)
+{
+	Game *game = player->game;
+
+	if (game->params->tournament_time > 0) {
+		/* if first player in and this is a tournament start the timer */
+		if (game->num_players == 1) {
+			game->tournament_countdown =
+			    game->params->tournament_time;
+			game->tournament_timer =
+			    g_timeout_add(game->tournament_countdown *
+					  tournament_minute + 500,
+					  &tournament_start_cb, game);
+			g_timeout_add(1000, &talk_about_tournament_cb,
+				      game);
+		} else {
+			if (game->tournament_timer != 0) {
+				player_send(player, FIRST_VERSION,
+					    LATEST_VERSION, "NOTE %s\n",
+					    N_
+					    ("This game will start soon."));
+			}
+		}
+	}
+}
+
 static gboolean mode_check_status(Player * player, gint event)
 {
 	StateMachine *sm = player->sm;
@@ -802,12 +945,14 @@ static gboolean mode_check_status(Player * player, gint event)
 	case SM_RECV:
 		if (sm_recv(sm, "status newplayer")) {
 			player_setup(player, -1, NULL, FALSE);
+			start_tournament_mode(player);
 			return TRUE;
 		}
 		if (sm_recv(sm, "status reconnect %S", &playername)) {
 			/* if possible, try to revive the player */
 			player_revive(player, playername);
 			g_free(playername);
+			start_tournament_mode(player);
 			return TRUE;
 		}
 		if (sm_recv(sm, "status newviewer")) {
@@ -955,7 +1100,8 @@ static void player_broadcast_internal(Player * player, BroadcastType type,
 	for (list = game->player_list; list != NULL;
 	     list = g_list_next(list)) {
 		Player *scan = list->data;
-		if ((scan->disconnected && !scan->sm->use_cache) || scan->num < 0
+		if ((scan->disconnected && !scan->sm->use_cache)
+		    || scan->num < 0
 		    || scan->version < first_supported_version
 		    || scan->version > last_supported_version)
 			continue;
